# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.http import (
    Http404,
    HttpResponseRedirect,
    HttpResponsePermanentRedirect,
)

from menus.utils import set_language_changer
from parler.views import ViewUrlMixin


class CMSLanguageChangerMixin(object):
    """Just adds support for the language changer in CMS."""
    def get(self, request, *args, **kwargs):
        response = super(CMSLanguageChangerMixin, self).get(
            request, *args, **kwargs)
        # We're assured to have self.object set after the call to super.
        set_language_changer(request, self.object.get_absolute_url)
        return response


class CanonicalUrlMixin(ViewUrlMixin):
    """
    Provides configurable control over how non-canonical URLs to views are
    handled. A view can specify by setting 'non_canonical_url_response_type' to
    one of 200, 301, 302 or 404. By default, handling will be to temporarily
    redirect to the canonical URL.
    """
    non_canonical_url_response_type = 302

    def get_non_canonical_url_response_type(self):
        response_type = getattr(self, "non_canonical_url_response_type", None)
        if response_type and response_type in [200, 301, 302, 404]:
            return response_type
        else:
            return 302

    def get(self, request, *args, **kwargs):
        """
        On GET, if the URL used is not the correct one, handle according to
        preferences by either:
            Allowing (200),
            Temprarily redirecting (302),
            Permanently redirecting (301) or
            Failing (404).
        """
        if not hasattr(self, 'object'):
            self.object = self.get_object()
        url = self.get_view_url()
        response_type = self.get_non_canonical_url_response_type()
        if (response_type == 200 or request.path == url):
            return super(CanonicalUrlMixin, self).get(
                request, *args, **kwargs)
        if response_type == 302:
            return HttpResponseRedirect(url)
        elif response_type == 301:
            return HttpResponsePermanentRedirect(url)
        else:
            raise Http404('This is not the canonical uri of this object.')



# -*- coding: utf-8 -*-

from django import forms


class BaseKnob(forms.widgets.NumberInput):
    #
    # Creates a widget out of jQuery-Knob:
    # https://github.com/aterrien/jQuery-Knob/
    #
    class Media:
        js = (
            'js/pinogy_common/jquery.knob.js',
            'js/pinogy_common/knob.js',
        )

    def render(self, name, value, attrs=None):
        attrs = {
            "class": "knob",
            "data-fgColor": "#029eee",
            "data-displayPrevious": "true",
        }
        return super(BaseKnob, self).render(name, value, attrs)


class PercentageKnob(BaseKnob):
    def _format_value(self, value):
        """
        Overrides the super method to avoid localization (JS will barf!).
        """
        if value is None or value is "":
            return ""
        return str(value)

    def render(self, name, value, attrs=None):
        attrs = {
            "data-min": "0",
            "data-max": "100",
        }
        return super(PercentageKnob, self).render(name, value, attrs)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ungettext, ugettext as _

from cms.utils.i18n import get_current_language
from cms.utils.urlutils import admin_reverse


class AdminActionBase:
    def message_user(self, modeladmin, request, model, count, state):
        msg = ungettext(
            "%(count)d %(obj_name)s was set to %(state)s.",
            "%(count)d %(obj_name)s were set to %(state)s.",
            count) % {
                'count': count,
                'obj_name': model._meta.verbose_name_raw,
                'state': state}
        modeladmin.message_user(request, msg)


class MakePublished(AdminActionBase):
    def __init__(self, is_true, obj_name_plural):
        self.is_true = is_true
        self.obj_name_plural = obj_name_plural

    def __call__(self, modeladmin, request, queryset):
        model = queryset.model
        count = queryset.update(is_published=self.is_true)
        self.message_user(modeladmin, request, model, count, "published")

    @property
    def __name__(self):
        if self.is_true:
            return "make_published"
        else:
            return "make_not_published"

    @property
    def short_description(self):
        return _("Mark selected {0} as {1}").format(
            MakeFeatured.obj_name_plural,
            "not published" if MakeFeatured.is_true else "published",
        )


class MakeFeatured(AdminActionBase):
    def __init__(self, is_true, obj_name_plural):
        self.is_true = is_true
        self.obj_name_plural = obj_name_plural

    def __call__(self, modeladmin, request, queryset):
        model = queryset.model
        count = queryset.update(is_featured=self.is_true)
        self.message_user(modeladmin, request, model, count, "featured")

    @property
    def __name__(self):
        if self.is_true:
            return "make_featured"
        else:
            return "make_not_featured"

    @property
    def short_description(self):
        return _("Mark selected {0} as {1}").format(
            MakeFeatured.obj_name_plural,
            "not featured" if MakeFeatured.is_true else "featured",
        )


class LinkedRelatedInlineMixin(object):
    #
    # This InlineAdmin mixin links the first field to the row object's own admin
    # change form.
    #
    # NOTE: If the first field is editable, it is undefined what will happen.
    # For best results, consider making all fields readonly (since they can be
    # edited with ease by following the link), and disabling the ability to add
    # new objects by overriding has_add_permission() to always return false.
    #
    extra = 0

    class ReverseLink:
        def __init__(self, display_link="link"):
            self.display_link = display_link

        def __call__(self, obj):
            model_name = obj.__class__.__name__.lower()
            admin_link = admin_reverse("{app_label}_{model_name}_change".format(
                app_label=obj._meta.app_label.lower(),
                model_name=model_name,
            ), args=(obj.id, ))
            return '<a href="{admin_link}" title="{title}">{link}</a>'.format(
                admin_link=admin_link,
                title=_('Click to view or edit this {0}').format(model_name),
                link=getattr(obj, self.display_link))

        @property
        def short_description(self):
            return self.display_link

        @property
        def allow_tags(self):
            return True

    def __init__(self, parent_model, admin_site):
        self.original_fields = self.get_fields_list(None)
        if len(self.original_fields):
            self.fields = ["reverse_link", ] + self.original_fields[1:]
        else:
            self.fields = "reverse_link"
        self.reverse_link = LinkedRelatedInlineMixin.ReverseLink(
            self.original_fields[0])
        super(LinkedRelatedInlineMixin, self).__init__(parent_model, admin_site)

    def get_fields_list(self, request, obj=None):
        """
        Returns a list of the AdminModel's declared `fields`, or, constructs it
        from the object, then, removes any `exclude`d items.
        """
        # ModelAdmin.get_fields came in Django 1.7, I believe
        if hasattr(super(LinkedRelatedInlineMixin, self), "get_fields"):
            fields = super(
                LinkedRelatedInlineMixin, self).get_fields(request, obj)
        elif self.fields:
            fields = self.fields
        else:
            fields = [f.name for f in self.model._meta.local_fields]
        if fields and self.exclude:
            fields = [f for f in fields if f not in self.exclude]
        if fields:
            return list(fields)
        else:
            return None

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(
            LinkedRelatedInlineMixin, self).get_readonly_fields(request, obj)
        if "reverse_link" not in readonly_fields:
            readonly_fields = list(readonly_fields) + ["reverse_link", ]
        # We want all fields to be readonly for this inline
        return readonly_fields


class AllTranslationsMixin(object):
    class Media:
        css = {'all': ('css/admin/all-translations-mixin.css', ), }

    def all_translations(self, obj):
        """
        Adds a property to the list_display that lists all translations with
        links directly to their change forms. Includes CSS to style the links to
        looks like tags with color indicating current language, active and
        inactive translations.

        A similar capability is in HVAD, and now there is this for
        Parler-based projects.
        """
        available = list(obj.get_available_languages())
        current = get_current_language()
        langs = []
        for code, title in settings.LANGUAGES:
            classes = ["lang-code", ]
            if code == current:
                classes += ["current", ]
            if code in available:
                classes += ["active", ]
                title += " (translated)"
            else:
                title += " (untranslated)"
            change_form_url = admin_reverse(
                '{app_label}_{model_name}_change'.format(
                    app_label=obj._meta.app_label.lower(),
                    model_name=obj.__class__.__name__.lower(),
                ), args=(obj.id, )
            )
            link = ('<a class="{classes}" href="{url}?language={code}" ' +
                'title="{title}">{code}</a>').format(
                classes=' '.join(classes),
                url=change_form_url,
                code=code,
                title=title,
            )
            langs.append(link)
        return ''.join(langs)
    all_translations.short_description = 'Translations'
    all_translations.allow_tags = True

    def get_list_display(self, request):
        """
        Unless the the developer has already placed "all_translations" in the
        list_display list (presumably specifically where she wants it), append
        the list of translations to the end.
        """
        list_display = super(
            AllTranslationsMixin, self).get_list_display(request)
        if 'all_translations' not in list_display:
            list_display = list(list_display) + ['all_translations', ]
        return list_display



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import models
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import get_language_from_request, ugettext as _

from parler.models import TranslatableModel


class EditModeMixin(object):
    """
    Super simple mixin that provides an `is_edit_mode` method that returns True
    if the current mode is the Edit mode.
    """
    def is_edit_mode(self, context):
        if context:
            request = context.get('request', None)
            toolbar = getattr(request, "toolbar", False)
            return (toolbar and toolbar.edit_mode)
        else:
            return False


class ViewObjectMixin(object):
    """
    This is very similar to Django's own DetailView, but for django CMS plugins.
    What we're doing here, is ultimately providing a `get_object` method that
    introspects the request object to determine which object should be returned.

    Key difference is that the `get_object` method relies on parameters, rather
    than class attributes to do its job. This allows the same methods to work
    for retrieving different objects in the same plugin.

    IMPORTANT NOTE: These method names differ from the CBV methods to prevent
    clobbering existing internal django CMS method names: `get_queryset` and
    `get_object` in particular.
    """

    def get_object_slug_field(self, request=None, slug_field=None):
        """
        Returns the name of the slug field to be used. Override as required for
        more dynamic cases.
        """
        return slug_field

    def get_object_queryset(self, request=None, model=None):
        """
        Get the queryset to look an object up against. May not be called if
        `get_object` is overridden. Override as required for more dynamic cases.
        """
        if model:
            return model.objects.all()
        else:
            raise ImproperlyConfigured("Did not receive a queryset. Pass "
                "`model`, `queryset`, or override get_object_queryset().")

    def get_view_object(self, request, queryset=None, model=None,
                        pk_url_kwarg=None, slug_field="slug",
                        slug_url_kwarg=None):
        """
        Given the paramaters provided, inspect the request object's
        `resolver_match` object to identify the desired type of object, look it
        up from the database and return it.
        """
        if queryset is None:
            queryset = self.get_object_queryset(request, model)

        if request and request.resolver_match:
            kwargs = request.resolver_match.kwargs
            pk = kwargs.get(self.pk_url_kwarg, None)
            slug = kwargs.get(self.slug_url_kwarg, None)
            if pk is not None:
                queryset = queryset.filter(pk=pk)
                return queryset.first()
            if slug is not None:
                slug_field = self.get_object_slug_field(
                    request, slug_field)
                queryset = queryset.filter(**{"slug": slug_field})
                return queryset.first()
        return None


class ParlerViewObjectMixin(ViewObjectMixin):
    """
    Very similar to the TranslatableSlugMixin views helper mixin from django-
    parler, this mixin enables ViewObjectPluginMixin to work with Parler-
    translated objects.

    Since CMS Plugins may use this mixin to find objects whose lookup fields
    (slugs) could be translated or not, this mixin may pass control back to
    ViewObjectMixin again if the field it is looking up doesn't appear
    translated.
    """
    def get_language(self, request):
        return get_language_from_request(request)

    def _has_model_field(self, model, field_name):
        try:
            model._meta.get_field_by_name(field_name)
            return True
        except models.FieldDoesNotExist:
            return False

    def get_view_object(self, request, queryset=None, model=None,
                        pk_url_kwarg=None, slug_field="slug",
                        slug_url_kwarg=None):
        """
        Overrides the get_object method of ViewObjectPluginMixin to work with
        translated slugs.
        """

        slug_field = self.get_object_slug_field(request, slug_field)

        # Check that this is a parler.models.TranslatableModel model we're
        # working with. If not, pass control back to this mixin's super class,
        # which can handle non-translated objects fine.

        # TODO: Should also check that this model has a translations property,
        # which "contains" a field whose name matches `slug_field`.
        if not issubclass(model, TranslatableModel):
            return super(ParlerViewObjectMixin, self).get_view_object(request,
                queryset, model, pk_url_kwarg, slug_field, slug_url_kwarg)

        if queryset is None:
            queryset = self.get_object_queryset(request, model)

        if request and request.resolver_match:
            kwargs = request.resolver_match.kwargs
            pk = kwargs.get(pk_url_kwarg, None)
            if pk is not None:
                return super(
                    ParlerViewObjectMixin, self).get_object(request, queryset)
            slug = kwargs.get(slug_url_kwarg, None)
            if slug is not None:
                queryset = queryset.active_translations(
                    language_code=self.get_language(request),
                    **{slug_field: slug})
                return queryset.first()



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from pinogy_common import __version__

REQUIREMENTS = [
    "django-parler",
]

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

setup(
    name='pinogy-common',
    version=__version__,
    description='Common utilities and code for Pinogy projects.',
    author='Pinogy',
    author_email='info@pinogy.com',
    url='https://bitbucket.org/pinogycorp/pgy-django-common',
    packages=find_packages(),
    license='LICENSE.txt',
    platforms=['OS Independent'],
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    include_package_data=True,
    zip_safe=False,
    test_suite="test_settings.run",
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from cms.utils.i18n import get_current_language, get_fallback_languages


class TranslationHelperMixin(object):

    def known_translation_getter(self, field, default=None, language_code=None,
                                 any_language=False):
        """
        This is meant to act like HVAD/Parler's safe_translation_getter() but
        respects the fallback preferences as defined in `settings.CMS_LANGUAGES`
        and returns both the translated field value and the language it
        represents as a tuple: (value, language).

        If no suitable language is found, then it returns (default, None)
        """
        # NOTE: We're using the CMS fallbacks here, rather than the Parler
        # fallbacks, the developer should ensure that their project's Parler
        # settings match the CMS settings.
        try:
            object_languages = self.get_available_languages()
            assert hasattr(object_languages, '__iter__')
        except [KeyError, AssertionError]:
            raise ImproperlyConfigured("TranslationHelperMixin must be used "
                "with a model defining get_available_languages() that returns "
                "a list of available language codes. E.g., django-parler's "
                "TranslatableModel.")

        language_code = language_code or get_current_language()
        site_id = getattr(settings, 'SITE_ID', None)
        languages = [language_code] + get_fallback_languages(
            language_code, site_id=site_id)

        # Grab the first language that is common to our list of fallbacks and
        # the list of available languages for this object.
        if languages and object_languages:
            language_code = next(
                (lang for lang in languages if lang in object_languages), None)

            if language_code:
                value = self.safe_translation_getter(field,
                    default=default, language_code=language_code)
                return (value, language_code)

        # No suitable translation exists
        return (default, None)
